import { from } from 'most';
import { subject } from './subject';
import { toTag } from './id';
import { PING, createHaltAction } from './config';
import { safeDispatch } from './port';
import { EventConstructor } from './event';
import { EffectConstructor } from './effect';
export function createDomain(store, domainName = '') {
  var redux = store;
  var config = redux.dispatch({
    type: PING
  }).payload;

  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var state$ = from(redux).map(() => store.getState()).multicast();
  return DomainConstructor(domainName, redux.dispatch, store.getState, state$, new Map(), addPlainHandler);
}
export function createRootDomain(domainName = '') {
  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var redux;
  var reduxStore;
  var config = {
    plain: new Map()
  };

  var dispatch = function dispatch(data) {
    return redux.dispatch(data);
  };

  function getState() {
    return reduxStore.getState();
  }

  var state$ = subject();
  var unsubscribe;

  function disablePrevious(previousDispatch) {
    previousDispatch(createHaltAction());

    if (typeof unsubscribe === 'function') {
      unsubscribe();
    }
  }

  function register(store) {
    var previousDispatch = redux != null ? redux.dispatch : v => v;
    redux = reduxStore = store;
    var pong = redux.dispatch({
      type: PING,
      payload: undefined
    });

    if (pong) {
      var payload = pong.payload;
      var plain = payload.plain;
      var oldPlain = config.plain;
      oldPlain.clear(); // for (const [key, val] of oldPlain.entries()) {
      //   // if (!plain.has(key)) {
      //   plain.set(key, val)
      //   // }
      // }

      config.plain = plain;
    }

    disablePrevious(previousDispatch); //$off

    var unsub = from(store).map(() => getState()).subscribe({
      next(value) {
        state$.next(value);
      },

      error(err) {},

      complete() {}

    });

    unsubscribe = () => unsub.unsubscribe();
  }

  var domain = DomainConstructor(domainName, dispatch, getState, state$, new Map(), addPlainHandler);
  domain.register = register;
  return domain;
}

function optsReducer([key, value]) {
  if (!(key in this)) return;

  var newVal = () => value;

  newVal;
  this[key] = newVal;
}

function mergeEffectOpts(defaults, opts) {
  if (opts === undefined) return Object.assign({}, defaults);

  if (!(typeof opts === 'object' && opts != null)) {
    throw new Error(`effect options should be object, got ${typeof opts}`);
  }

  var resultObj = Object.assign({}, defaults);
  Object.keys(
  /*::Object.freeze*/
  opts).map(key => [key,
  /*::Object.freeze*/
  opts[key]]).forEach(optsReducer, resultObj);
  return resultObj;
}

function DomainConstructor(domainName, dispatch, getState, state$, events = new Map(), addPlainHandler = () => {}, configGetter) {
  var getConfig;

  if (typeof configGetter !== 'function') {
    var config = {
      unused: () => 'warn',
      watchFailCheck: () => 'warn',
      dispatch
    };

    getConfig = () => config;
  } else getConfig = configGetter;

  var disp = getDispatch(getConfig);
  var effectOpts = {
    unused() {
      return getConfig().unused();
    },

    watchFailCheck() {
      return getConfig().watchFailCheck();
    },

    dispatch: disp
  };

  function recoverable(events$) {
    var str$ = events$.multicast();

    var failureHandler = e => {
      portFailWarn({
        domainName
      }, e);
      return str$;
    };

    return str$.recoverWith(failureHandler);
  }

  return {
    port(events$) {
      return recoverable(events$).observe(data => {
        safeDispatch(data, dispatch);
      });
    },

    register(store) {
      console.warn(`Not implemented`);
    },

    effect(name, opts) {
      var optsFull = mergeEffectOpts(effectOpts, opts);
      return EffectConstructor(domainName, dispatch, getState, state$, events, name, optsFull);
    },

    domain(name) {
      return DomainConstructor(toTag(domainName, name), dispatch, getState, state$, events, addPlainHandler, getConfig);
    },

    event(name) {
      return EventConstructor(domainName, dispatch, getState, state$, events, name);
    },

    typeConstant(name) {
      var action$ = subject();
      addPlainHandler(name, data => {
        if (data.meta == null) data.meta = {};
        data.meta.plain = true;
        var resultEvent = result(data.payload);
        resultEvent.meta.passed = true;
        resultEvent.meta.plain = true;
        action$.next(data.payload);
        getConfig().dispatch(resultEvent);
      });
      var result = EventConstructor('', dispatch, getState, state$, events, name, action$, {
        isPlain: true,
        watchFailCheck: effectOpts.watchFailCheck()
      });
      return result;
    }

  };
}

function getDispatch(getConfig) {
  var dispatch = value => {
    var result = getConfig().dispatch(value);
    return result;
  };

  return dispatch;
}

function portFailWarn({
  domainName
}, error) {
  console.warn(`


  Port stream should not fail.
  Next failure will throw.

  Domain:
    ${domainName}
  Error:
    ${error}

`);
}
//# sourceMappingURL=domain.js.map