"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EffectConstructor = EffectConstructor;

var _subject = require("../subject");

var _id = require("../id");

var _event = require("../event");

var _port = require("../port");

var _eventCommon = require("../event-common");

var _warn = require("./warn");

var _watch = require("./watch");

function EffectConstructor(domainName, dispatch, getState, state$, events, name, opts, action$ = (0, _subject.subject)()) {
  var handlers = new Set();
  var {
    eventID,
    nextSeq,
    getType
  } = (0, _eventCommon.basicCommon)(domainName, name);
  handlers.add((payload, state) => action$.next(payload));
  var using = getWarn({
    domainName,
    name,
    mode: () => opts.unused()
  });

  function use(thunk) {
    using = thunk;
  }

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: (0, _id.nextPayloadID)(),
      eventID,
      seq: nextSeq(),
      passed: false
    }
  });

  var effect = params => {
    var run = () => {
      run = () => {};

      var runned = runUsing(params, using);
      runned.then(result => {
        var data = {
          params,
          result
        };
        dispatcher(done(data), dispatch);
        resolve(data);
      }, error => {
        var data = {
          params,
          error
        };
        dispatcher(fail(data), dispatch);
        reject(data);
      });
    };

    var runner = dispatchHook => {
      if (canDispatch) {
        canDispatch = false;
        dispatcher(result, dispatch, dispatchHook);

        for (var handler of handlers) {
          handler(params, getState());
        }

        run(); // action$.next(params)
      }
    };

    var resolve = val => {};

    var reject = val => {};

    var doneP = new Promise(rs => resolve = rs);
    var failP = new Promise(rs => reject = rs);
    var toSend = Promise.resolve(params);
    var result = create(params);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        runner(dispatchHook);
        return toSend;
      },

      done() {
        runner();
        return doneP;
      },

      fail() {
        runner();
        return failP;
      },

      promise() {
        runner();
        return Promise.race([doneP, failP.then(data => Promise.reject(data))]);
      }

    });
  };

  var done = (0, _event.EventConstructor)(domainName, dispatch, getState, state$, events, [name, 'done'].join(' '));
  var fail = (0, _event.EventConstructor)(domainName, dispatch, getState, state$, events, [name, 'fail'].join(' '));
  effect.use = use;
  effect.getType = getType;
  setToString(effect, getType);
  effect.done = done;
  effect.fail = fail;
  effect.epic = (0, _port.port)(dispatch, state$, action$);
  effect.watch = (0, _watch.createWatcher)({
    event: effect,
    domainName,
    name,
    opts
  });

  effect.trigger = function trigger(query, eventName = 'trigger') {
    var triggerEvent = (0, _event.EventConstructor)(domainName, dispatch, getState, state$, events, [name, eventName].join(' '));
    triggerEvent.watch((_, state) => {
      var queryResult = query(state);
      return effect(queryResult);
    });
    return triggerEvent;
  };

  effect.subscribe = subscriber => action$.subscribe(subscriber);

  (0, _eventCommon.observable)(effect, action$);
  return effect;
}

function setToString(effect, getType) {
  effect.toString = getType;
}

function runUsing(params, using) {
  var response;

  try {
    response = using(params);
  } catch (err) {
    return Promise.reject(err);
  }

  if (typeof response === 'object' && response != null && typeof response.then === 'function') {
    return response;
  }

  return Promise.resolve(response);
}

function dispatcher(value, dispatchDefault, dispatchHook, isBatch = false) {
  if (!isAction(value)) {
    return value;
  }

  var dispatch = typeof dispatchHook === 'function' ? dispatchHook : dispatchDefault;
  var result = dispatch(value);
  return result;
}

function isAction(data) {
  return typeof data === 'object' && data != null && typeof data.type === 'string';
}

function watchFailWarn({
  domainName,
  name,
  mode
}, error) {
  switch (mode) {
    case 'throw':
      {
        throw error;
      }

    case 'warn':
      {
        (0, _warn.watchWarn)(domainName, name, error);
      }

    case 'off':
    default:
      {
        return;
      }
  }
}

var getWarn = ({
  domainName,
  name,
  mode
}) => function never(props) {
  switch (mode()) {
    case 'throw':
      {
        return Promise.reject((0, _warn.implError)(domainName, name, props));
      }

    case 'warn':
      {
        (0, _warn.implWarn)(domainName, name, props);
      }

    case 'off':
    default:
      {
        return new Promise(() => {});
      }
  }
};
//# sourceMappingURL=effect.js.map