"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EventConstructor = EventConstructor;

var _most = require("most");

var _subject = require("./subject");

var _id = require("./id");

var _port = require("./port");

var _eventCommon = require("./event-common");

var _warn = require("./effect/warn");

var _watch = require("./effect/watch");

// import type {Store} from 'redux'
function EventConstructor(domainName, dispatch, getState, state$, events, name, action$ = (0, _subject.subject)(), config = {
  isPlain: false,
  watchFailCheck: 'warn'
}) {
  var {
    eventID,
    nextSeq,
    getType
  } = (0, _eventCommon.basicCommon)(domainName, name);
  var handlers = new Set();
  if (!config.isPlain) handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: (0, _id.nextPayloadID)(),
      eventID,
      seq: nextSeq(),
      passed: false,
      plain: config.isPlain
    }
  });

  function eventInstance(payload) {
    var toSend = Promise.resolve(payload);
    var result = create(payload);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        if (canDispatch) {
          var dispatcher = dispatchHook ? dispatchHook : dispatch;
          canDispatch = false;
          dispatcher(result);

          for (var handler of handlers) {
            handler(payload, getState());
          } // action$.next(payload)

        }

        return toSend;
      }

    });
  }

  eventInstance.getType = getType; //$off

  eventInstance.toString = getType;
  eventInstance.watch = (0, _watch.createWatcher)({
    event: eventInstance,
    domainName,
    name,
    opts: {
      watchFailCheck: () => config.watchFailCheck
    }
  });
  eventInstance.epic = (0, _port.port)(dispatch, state$, action$);

  eventInstance.subscribe = subscriber => action$.subscribe(subscriber); // eventInstance.port = function port<R>(events$: Stream<R>) {
  //   return events$.observe(data => safeDispatch(data, dispatch))
  // }


  eventInstance.trigger = function trigger(query, eventName = 'trigger') {
    var triggerEvent = EventConstructor(domainName, dispatch, getState, state$, events, [name, eventName].join(' '));
    triggerEvent.watch((_, state) => {
      var queryResult = query(state);
      return eventInstance(queryResult);
    });
    return triggerEvent;
  };

  (0, _eventCommon.observable)(eventInstance, action$);
  events.set(getType(), eventInstance);
  return eventInstance;
}
//# sourceMappingURL=event.js.map