"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createReducer = createReducer;

function normalizeType(typeOrActionCreator) {
  if (typeOrActionCreator && typeOrActionCreator.getType) {
    return typeOrActionCreator.toString();
  }

  return typeOrActionCreator;
}

function createReducer(defaultState, handlers = {}, thisType) {
  var opts = {
    fallback: null,
    defaultState
  };
  var reducer = Object.assign(reduce, {
    has,
    on,
    off,
    options,
    reset
  });
  var returnType = thisType ? thisType : reducer;

  function has(typeOrActionCreator) {
    return !!handlers[normalizeType(typeOrActionCreator)];
  }

  function reset(typeOrActionCreator) {
    on(typeOrActionCreator, (state, payload, meta) => opts.defaultState);
    return returnType;
  }

  function on(typeOrActionCreator, handler) {
    if (Array.isArray(typeOrActionCreator)) {
      typeOrActionCreator.forEach(action => {
        on(action, handler);
      });
    } else {
      handlers[normalizeType(typeOrActionCreator)] = handler;
    }

    return returnType;
  }

  function off(typeOrActionCreator) {
    if (Array.isArray(typeOrActionCreator)) {
      typeOrActionCreator.forEach(off);
    } else {
      delete handlers[normalizeType(typeOrActionCreator)];
    }

    return returnType;
  }

  function options(newOpts) {
    // $off
    Object.keys(newOpts).forEach(name => opts[name] = newOpts[name]);
    return returnType;
  }

  if (typeof handlers === 'function') {
    var factory = handlers; // $off

    handlers = {};
    factory(on, off);
  }

  function reduce(stateRaw, action) {
    var state = stateRaw === undefined ? opts.defaultState : stateRaw;

    if (!action || typeof action.type !== 'string') {
      return state;
    }

    if (action.type.startsWith('@@redux/')) {
      return state;
    }

    var handler = handlers[String(action.type)] || opts.fallback;

    if (handler) {
      var result = handler(state, action.payload, action.meta);
      if (result === undefined) return opts.defaultState;
      return result;
    }

    return state;
  }

  return reducer;
}
//# sourceMappingURL=reducer.js.map